#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import math
import time
import numpy as np
import matplotlib.pyplot as plt
from collections import deque

sys.path.append('environment')
sys.path.append('samplingplanner')
 
import Environment as env
import SamplingPlanner as sp
import PRMPlanner as prm



if __name__ == "__main__":
    #define the planning scenarios
    #scenario name
    #start configuration
    #goal configuration
    #limits for individual DOFs
    #sampling steps

    scenarios_prm = [("environments/simple_test", (2,2,0,0,0,0), (-2,-2,0,0,0,0), [(-3,3), (-3,3), (0,0), (0,0), (0,0), (0,0)], 300),
                     ("environments/simple_test", (2,2,0,0,0,0), (-2,-2,0,0,0,math.pi/2), [(-3,3), (-3,3), (0,0), (0,0), (0,0), (0,2*math.pi)], 1000),
                     ("environments/squares", (1,-3,0,0,0,0), (4,4,0,0,0,0), [(0,5), (-5,6), (0,0), (0,0), (0,0), (0,2*math.pi)], 2500),
                     ("environments/cubes", (2.5,-3,-2.5,0,0,3*math.pi/2), (2.5,4,-2.5,0,math.pi/2,3*math.pi/2), [(2,3), (-5,6), (-3,-2), (0,2*math.pi), (0,2*math.pi), (0,2*math.pi)], 10000),
                   ]

    #####################################
    ## EVALUATION OF THE PRM PLANNER
    #####################################
    for scenario in scenarios_prm:
        name = scenario[0]
        start = sp.te_2_se3(scenario[1])
        goal = sp.te_2_se3(scenario[2])
        limits = scenario[3]
        n_samples = scenario[4]
        
        print("processing scenario: " + name)

        #initiate environment and robot meshes
        environment = env.Environment()
        environment.load_environment(name)
        ax = environment.plot_environment(start, ax=None, limits=limits)
        plt.pause(0.00001)

        #instantiate the planner
        planner = prm.PRMPlanner(limits)
    
        #plan the path through the environment
        path = planner.plan(environment, start, goal, n_samples = n_samples)

        ax = environment.plot_environment(path[0], ax=ax,  limits=limits)
        planner.plot_samples(plot_samples = False, plot_edges = False)
        for i in range(len(path)-1):
            print ('%5d / %5d' % (i,len(path)))

            se3_1 = path[i]
            se3_2 = path[i+1]
            for se3_j in planner.create_sub_path(se3_1,se3_2):
                ax = environment.plot_environment(se3_j, ax=ax,  limits=limits)
                planner.plot_samples(plot_samples = False, plot_edges = False)
                plt.pause(0.00001)

            if planner.is_valid_edge(se3_1,se3_2,environment,verbose=True):
                print ('ok')
            else:
                print ('fail')
                break
